
/**
 * Class for menage display on a family graph
 * @property {Vis.Network}                 network                     The vis network
 * @property {RequestLauncher}             requestLauncher             The request lautcher
 * @property {vis.Dataset}                 nodes                       Array of all nodes
 */
FamilyGraph = function()
{
  this.network = null;
  this.nodes = null;
  this.edges = null;

  this.requestLauncher = new RequestLauncher();
};

/**
 * Type of relation between person of the graph
 */
FamilyGraph.FATHER_CODE = 'P22';
FamilyGraph.MOTHER_CODE = 'P25';
FamilyGraph.SPOUSE_CODE = 'P26';
FamilyGraph.BROTHER_SISTER_CODE = 'P3373'; // brother or sister
FamilyGraph.CHILD_CODE = 'P40';

/**
 * Type of node in the graph
 */
FamilyGraph.NODE_TYPE_WOMAN = "http://www.wikidata.org/entity/Q6581072";
FamilyGraph.NODE_TYPE_MAN = "http://www.wikidata.org/entity/Q6581097";
FamilyGraph.NODE_TYPE_UNION = "UNION";

/**
 * Initialize the graph
 * @param {String}               uri                     URI of the item
 * @param {String}               rulerContent            Content of iten in the timeline
 * @param {Vis.DataSet}          timeLineItem            Dataset of items in the timeline
 */
FamilyGraph.prototype.init = function(uri, rulerContent, timeLineItems)
{
  let me = this;

  // Init label
  let rulerLabel = rulerContent.split("<")[1];
  rulerLabel = rulerLabel.split(">")[1];
  
  // create graph structure
  this.nodes = new vis.DataSet([]);
  this.edges = new vis.DataSet([]);
  let container = document.getElementById('graph');
  
  let data = {
    nodes: this.nodes,
    edges: this.edges
  };
  let options = {
    layout: {
        hierarchical : {
          enabled: true,
          direction : 'UD',
          sortMethod : 'hubsize',
          levelSeparation: 130,
          nodeSpacing: 150,
          treeSpacing: 200,
        }
    },
    physics:{
      enabled: true,
      solver: 'hierarchicalRepulsion',
      hierarchicalRepulsion : 
      {
        nodeDistance : 155
      }
    },
    nodes: {
      borderWidth: 2,
      size: 30,
      color: {
        border: "#cccccc",
        background: "#f0f0f0"
      },
      font: {
        color: "#000000",
        background : "#d7cff4",
        bold: { color: "#000000" },
      },
      shapeProperties: {
        useBorderWithImage: true
      }
    },
    edges: {
      color: { 
        color: "#53a686",
        highlight: "#4e7b6a",
      },
      width: 2
    },
  };

  let request = me.createRequest(uri, true, true);

  me.requestLauncher.launch(request, function(resultDatas)
  {
    if(resultDatas != null)
    {
      // Create node of the person 
      if(resultDatas.results.bindings.length > 0)
      {
        let data = resultDatas.results.bindings[0];
        let rulerGenre = data["rulerGenre"]["value"];

        me.addNode(uri, rulerLabel, rulerGenre, true, 2);
      }

      // Create node of others datas
      var levels = [];
      var relationsUri = [];
      for(let i = 0; i < resultDatas.results.bindings.length; i++)
      {
        let data = resultDatas.results.bindings[i];
        let level = me.createNodeData(data, uri, 2, timeLineItems);

        levels.push(level);
        relationsUri.push(data["relation"]["value"]);
      }

      me.network = new vis.Network(container, data, options);
    }
  });
};


/**
 * Create the request string
 * @param {String}               uri                     URI of the item
 */
FamilyGraph.prototype.createRequest = function(uri)
{
  let request = '';
  request += 'SELECT (<'+uri+'> as ?ruler) ?rulerGenre ?relation ?relationLabel ?relationType ?relationGenre ';
  request += 'WHERE ';
  request += '{ ';
  request += '  OPTIONAL { <'+uri+'> wdt:P21 ?rulerGenre } .  ';
  request += '  { <'+uri+'> wdt:P22 ?relation . VALUES ?relationType { "'+FamilyGraph.FATHER_CODE+'" } }  ';
  request += '  UNION { <'+uri+'> wdt:P25 ?relation . VALUES ?relationType { "'+FamilyGraph.MOTHER_CODE+'" }  }  ';
  request += '  UNION { <'+uri+'> wdt:P26 ?relation . VALUES ?relationType { "'+FamilyGraph.SPOUSE_CODE+'" }  }  ';
  request += '  UNION { <'+uri+'> wdt:P40 ?relation . VALUES ?relationType { "'+FamilyGraph.CHILD_CODE+'" }  } .  ';
  request += '  OPTIONAL { ?relation wdt:P21 ?relationGenre } .  ';
  request += '  SERVICE wikibase:label { bd:serviceParam wikibase:language "[AUTO_LANGUAGE],fr,en" }  ';
  request += '} ';

  return request;
};


/**
 * Add a new node in the graph
 * @param {Array}                data                  Array of getted data
 * @param {String}               uri                   URI of the node
 * @param {String}               label                 Label of the node
 * @param {Number}               level                 The display level of the node
 * @param {Vis.DataSet}          timeLineItem          Dataset of items in the timeline
 */
FamilyGraph.prototype.createNodeData = function(data, uri, level, timeLineItems)
{
  let me = this;

  let relationUri = data["relation"]["value"];

  // 
  let nodeInNetwork = this.nodes.get({
    filter: function (item) {
      return item.uri == relationUri;
    }
  });
  
  if(nodeInNetwork.length == 0)
  {
    // 
    let relationLabel = data["relationLabel"]["value"];
    let relationType = data["relationType"]["value"];
    let relationGenre = data["relationGenre"]["value"];

    if(relationType == FamilyGraph.FATHER_CODE || relationType == FamilyGraph.MOTHER_CODE)
    {
      level -= 1;
    }
    else if(relationType == FamilyGraph.SPOUSE_CODE || relationType == FamilyGraph.BROTHER_SISTER_CODE)
    {
      level += 0;
    }
    else
    {
      level += 1;
    }

    // Check if the current data exist in the timeline
    let itemsInTimeLine = timeLineItems.get({
      filter: function (item) {
        return item.uri == relationUri;
      }
    });
    let ruler = false;
    if(itemsInTimeLine.length > 0)
    {
        ruler = true;
    }

    me.addNode(relationUri, relationLabel, relationGenre, ruler, level);

    // Add edges
    me.edges.add(
    [
      {
        from: uri,
        to: relationUri
      }
    ]);
  }

  return level;
};

/**
 * Add a new node in the graph
 * @param {String}      uri             URI of the node
 * @param {String}      label           Label of the node
 * @param {String}      type            Type of the node
 * @param {Boolean}     ruler           True if in the ruler timeline 
 * @param {Number}      level           The display level of the node
 */
FamilyGraph.prototype.addNode = function(uri, label, type, ruler, level)
{
  let image = "img/ruler.png";
  let color = "#999999";
  let borderColor = "#cccccc";
  let size = 25;
  let shapeType = "image";

  if(type == FamilyGraph.NODE_TYPE_WOMAN)
  {
    if(ruler)
    {
      image = "img/queen.png";
    }
    else
    {
      image = "img/woman.png";
    }
    
    borderColor = "#d9a7ff";
  }
  else if(type == FamilyGraph.NODE_TYPE_MAN)
  {
    if(ruler)
    {
      image = "img/king.png";
    }
    else
    {
      image = "img/men.png";
    }

    borderColor = "#949cff";
  }
  else if(type == FamilyGraph.NODE_TYPE_UNION)
  {
    image = "img/king.png";
    borderColor = "#949cff";
    color = "yellow";
    shapeType = "dot"
    size = 10;
  }

  this.nodes.add([
    {
      id: uri,
      uri : uri,
      label: label,
      image: image,
      size : size,
      shape: shapeType,
      color: {
        border: borderColor,
      },
      level: level
    }
  ]);
};
